<?php

/**
 * @file
 * Administration forms and support functions of Visual Website Optimizer module
 *
 * Offload the large amount of admin only code to its own file so it does not
 * get loaded and parsed with every single page load.
 */

/**
 * Root Settings form
 */
function visual_website_optimizer_settings($form, &$form_state) {

  $settings = array_merge(
    _visual_website_optimizer_settings_defaults(),
    variable_get('visual_website_optimizer_settings', array())
  );

  $form['enabled'] = array(
    '#type' => 'radios',
    '#title' => t('Enable Visual Website Optimizer'),
    '#description' => t('To start using Visual Website Optimizer, this setting must be enabled. This setting can be used to globally disable VWO from working on your site without disabling yours tests on the VWO website or disabling the module. Leaving the module Disabled should only be a temporary measure.'),
    '#options' => array(
      'off' => t('Disabled'),
      'on' => t('Enabled'),
    ),
    '#default_value' => $settings['enabled'],
  );

  $form['id'] = array(
    '#type' => 'textfield',
    '#title' => t('VWO ID Number'),
    '#description' => t('Your numeric account ID. This is the number after <q>var _vis_opt_account_id = </q> in the VWO Tracking Code. ')
      . '<br /><strong>' . t('You can use the <a href="@url">Parse Account ID</a> tool to extract the Account ID from the full VWO Tracking Code.',
      array('@url' => url('admin/config/system/visual_website_optimizer/vwoid'))) . '</strong>',
    '#size' => 15,
    '#maxlength' => 20,
    '#required' => TRUE,
    '#default_value' => $settings['id'],
  );

  $form['advanced'] = array(
    '#type' => 'fieldset',
    '#title' => t('Advanced settings'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,

    'synchtype' => array(
      '#type' => 'radios',
      '#title' => t('Synchonosity'),
      '#description' => t('This new asynchronous version of Visual Website Optimizer code reduces page load time as the VWO code is downloaded in parallel to site code. It also ensures that your site is never slowed down even if the VWO servers are inaccessible.')
        . '<br />'
        . t('Note: VWO have extensively tested the asynchronous code across variety of browsers (including IE7) and it works perfectly. Though they are keeping this code in beta until they get feedback from users that it works and is indeed a significant improvement over existing Synchronous code.'),
      '#options' => array(
        'sync' => t('Synchronous (default, safe)'),
        'async' => t('Asynchronous (BETA, see note)'),
      ),
      '#default_value' => $settings['synchtype'],
    ),

    'asyncusejquery' => array(
      '#type' => 'radios',
      '#title' => t('Asynchronous jQuery setting'),
      '#description' => t('When using the Asynchronous <em>BETA</em> setting above, there is the option to turn on and off a setting to use the existing jQuery included in Drupal by default. This option defaults to using the VWO version of jQuery, however it is completely untested. Please provide feedback tothe module Author regarding this setting.'),
      '#options' => array(
        'local' => t('Use Drupal local jQuery'),
        'import' => t('Use VWO jQuery like functions'),
      ),
      '#default_value' => $settings['asyncusejquery'],
    ),
  );

  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save configuration'),
  );

  return $form;
}

/**
 * Validation for settigns form.
 * Verify that the account ID is numeric
 */
function visual_website_optimizer_settings_validate($form, &$form_state) {
  if (!preg_match('/^\d+$/', $form_state['values']['id'])) {
    form_set_error('id', t('Your Visual Website Optimizer ID should be numeric.'));
  }
}

/**
 * Submit handler for visual_website_optimizer_setup_form form.
 * Push all settings into one variable since they are always used together.
 */
function visual_website_optimizer_settings_submit($form, &$form_state) {
  // Start with defaults
  $settings = array_merge(
    _visual_website_optimizer_settings_defaults(),
    variable_get('visual_website_optimizer_settings', array())
  );

  foreach ($settings as $key => $value) {
    if (isset($form_state['values'][$key])) {
      $settings[$key] = $form_state['values'][$key];
    }
  }

  variable_set('visual_website_optimizer_settings', $settings);

  drupal_set_message(t('Visual Website Optimizer settings have been saved.'), 'status');
}

/**
 * Form to accept full cut'n'paste of Tracking Code from website and pull Account ID
 * from it using preg_match().
 */
function visual_website_optimizer_parse_account_id() {
  $form = array();

  $settings = array_merge(
    _visual_website_optimizer_settings_defaults(),
    variable_get('visual_website_optimizer_settings', array())
  );

  // Display current ID if it exists
  $form['current_id'] = array(
    '#type' => 'textfield',
    '#title' => t('Current Account ID'),
    '#disabled' => TRUE,
    '#default_value' => $settings['id'],
    '#size' => 15,
  );

  // Text area to paste into
  $form['parse_area'] = array(
    '#type' => 'textarea',
    '#title' => t('Paste full VWO Tracking Code here'),
    '#description' => t('The Tracking Code is available by logging into the Visual Website Optimizer website, selecting the "Tools" menu in the top right corner, and then "Get Tracking Code". Select all the text in the "VWO Tracking Code" text box and copy it here.'),
    '#rows' => '15',
  );

  // Submit buttons
  $form['parse'] = array(
    '#type' => 'submit',
    '#value' => t('Parse and Save Account ID from pasted javascript'),
  );

  return $form;
}

/**
 * Validation handler for visual_website_optimizer_parse_account_id().
 * Ensures that the the Account ID can be parsed from the pasted code and then stores it
 * for the sumbit handler.
 * Pull the code from either the sync or async code
 */
function visual_website_optimizer_parse_account_id_validate($form, &$form_state) {

  // Synchronous version of code
  if (preg_match('/var _vis_opt_account_id = ([[:digit:]]+);/', $form_state['values']['parse_area'], $matches)) {
    $form_state['values']['parsed_id'] = $matches[1];
  }

  // Asynchronous version of code
  else if (preg_match('/var account_id=([[:digit:]]+),/', $form_state['values']['parse_area'], $matches)) {
    $form_state['values']['parsed_id'] = $matches[1];
  }

  // Failure
  else {
    form_set_error('parse_area', t('Unable to locate Account ID in pasted code.'));
  }
}

/**
 * Submit handler for visual_website_optimizer_parse_account_id().
 * Saves the Account ID stored from validation handler.
 */
function visual_website_optimizer_parse_account_id_submit($form, &$form_state) {

  // Pull existing settings, adjust the Account ID, save them.
  $settings = array_merge(
    _visual_website_optimizer_settings_defaults(),
    variable_get('visual_website_optimizer_settings', array())
  );
  $settings['id'] = $form_state['values']['parsed_id'];
  variable_set('visual_website_optimizer_settings', $settings);

  drupal_set_message(t('Saved VWO Account ID parsed from pasted code: @id', array('@id' => $settings['id'])), 'status');

  $form_state['redirect'] = 'admin/config/system/visual_website_optimizer';
}

/**
 * Settings to configure the filters that determine when this javascript
 * should be added to the page in hook_init().
 */
function visual_website_optimizer_filters() {
  $form = array();

  $settings = array_merge(
    _visual_website_optimizer_settings_defaults(),
    variable_get('visual_website_optimizer_settings', array())
  );

  // User configuration options
  $form['userfilter'] = array(
    '#type' => 'fieldset',
    '#title' => t('User specific visibility settings'),
    '#collapsible' => TRUE,
    '#collapsed' => FALSE,

    'userradios' => array(
      '#type' => 'radios',
      '#title' => t('Custom visibility settings'),
      '#description' => t('Allow individual users to optin/out to being included in the tests in their account settings.'),
      '#options' => array(
        'nocontrol' => t('Users cannot control whether or not the javascript is added.'),
        'optout' => t('Include javascript by default, but let individual users turn it off.'),
        'optin' => t('Do not include javascript by default but let individual users turn it on.'),
      ),
      '#default_value' => $settings['userfilter'],
    ),
  );

  // Role options
  $role_options = user_roles(TRUE, NULL);
  $form['rolefitler'] = array(
    '#type' => 'fieldset',
    '#title' => t('Role specific visibility settings'),
    '#collapsible' => TRUE,

    'rolechecks' => array(
      '#type' => 'checkboxes',
      '#title' => t('Include VWO for specific roles'),
      '#description' => t('Include VWO javascript only for the selected role(s). If you select no roles, it will be included for all users.'),
      '#options' => $role_options,
      '#default_value' => $settings['rolefilter'],
    ),
  );

  // Page specific options
  $access = user_access('use PHP for filtering condition');
  if ($settings['pagefilter'] == 'usephp' && !$access) {
    $form['pagefilter'] = array(
      'pageradios' => array(
        '#type' => 'value',
        '#value' => 'usephp',
      ),
      'pagelist' => array(
        '#type' => 'value',
        '#value' => $settings['pagelist'],
      ),
    );
  }
  else {
    $page_options = array(
      'listexclude' => t('Show on every page except the listed pages.'),
      'listinclude' => t('Show on only the listed pages.'),
      'usephp' => t('Show if the following PHP code returns <code>TRUE</code> (PHP-mode, experts only).'),
    );
    if (!$access) unset($page_options['usephp']);

    $form['pagefilter'] = array(
      '#type' => 'fieldset',
      '#title' => t('Page specific visibility settings'),
      '#collapsible' => TRUE,

      'pageradios' => array(
        '#type' => 'radios',
        '#title' => t('Include VWO javascript on specific pages'),
        '#options' => $page_options,
        '#default_value' => $settings['pagefilter'],
      ),

      'pagelist' => array(
        '#type' => 'textarea',
        '#title' => t('Pages'),
        '#description' => t("Enter one page per line as Drupal paths. The '*' character is a wildcard. Example paths are %blog for the blog page and %blog-wildcard for every personal blog. %front is the front page.", array('%blog' => 'blog', '%blog-wildcard' => 'blog/*', '%front' => '<front>')),
        '#default_value' => $settings['pagelist'],
      ),
    );
    if ($access) $form['pagefilter']['pagelist']['#description'] .= ' '. t('If the PHP-mode is chosen, enter PHP code between %php. Note that executing incorrect PHP-code can break your Drupal site.', array('%php' => '<?php ?>'));
  }

  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save settings'),
  );

  return $form;
}

/**
 * Submit handler for visual_website_optimizer_filters().
 * Start from defaults and add new settings.
 */
function visual_website_optimizer_filters_submit($form, &$form_state) {
  $settings = array_merge(
    _visual_website_optimizer_settings_defaults(),
    variable_get('visual_website_optimizer_settings', array())
  );

  if (isset($form_state['values']['userradios'])) {
    $settings['userfilter'] = $form_state['values']['userradios'];
  }
  else {
    $settings['userfilter'] = 'nocontrol';
  }

  // Only store the roles that are checked.
  if (isset($form_state['values']['rolechecks'])) {
    $only_true_roles = array_filter($form_state['values']['rolechecks']);
    $settings['rolefilter'] = $only_true_roles;
  }
  else {
    $settings['rolefilter'] = array();
  }

  // Page listing
  if (isset($form_state['values']['pageradios']) && isset($form_state['values']['pagelist'])) {
    $settings['pagefilter'] = $form_state['values']['pageradios'];
    $settings['pagelist'] = $form_state['values']['pagelist'];
  }
  else {
    $settings['pagefilter'] = 'listexclude';
    $settings['pagelist'] = '';
  }

  variable_set('visual_website_optimizer_settings', $settings);

  drupal_set_message(t('Visual Website Optimizer settings have been saved.'), 'status');
}

